@@ -34,6 +34,7 @@ from weka.filters import Filter
 
 # logging setup
 logger = logging.getLogger("weka.clusterers")
+logger.setLevel(logging.INFO)
 
 
 class Clusterer(OptionHandler):
@@ -62,7 +63,7 @@ class Clusterer(OptionHandler):
 
     def _make_calls(self):
         """
-        Method for generating instances using javabridge.make_call.
+        Method for obtaining method instances for faster access.
         Members must start with "_mc_"
         """
         super(Clusterer, self)._make_calls()
@@ -530,7 +531,7 @@ def main(args=None):
 
     jvm.start(jars, max_heap_size=parsed.heap, packages=True)
 
-    logger.debug("Commandline: " + join_options(sys.argv[1:]))
+    logger.info("Commandline: " + join_options(sys.argv[1:]))
 
     try:
         clusterer = Clusterer(classname=parsed.clusterer)
