@@ -82,7 +82,7 @@ def add_system_classpath(cp):
 def start(class_path=None, bundled=True, packages=False, system_cp=False, max_heap_size=None, system_info=False,
           auto_install=False, logging_level=logging.DEBUG):
     """
-    Initializes the javabridge connection (starts up the JVM).
+    Initializes the jpype connection (starts up the JVM).
 
     :param class_path: the additional classpath elements to add
     :type class_path: list
@@ -129,9 +129,16 @@ def start(class_path=None, bundled=True, packages=False, system_cp=False, max_he
         add_system_classpath(full_cp)
 
     logger.debug("Classpath=" + str(full_cp))
-    logger.debug("MaxHeapSize=" + ("default" if (max_heap_size is None) else max_heap_size))
 
     args = []
+
+    # heap size
+    if max_heap_size is not None:
+        logger.debug("MaxHeapSize=%s" % max_heap_size)
+        args.append("-Xmx%s" % max_heap_size)
+    else:
+        logger.debug("MaxHeapSize=default")
+
     weka_home = None
     if packages is not None:
         if isinstance(packages, bool):
