@@ -16,17 +16,24 @@
 
 import argparse
 import json
+import logging
 import sys
 import traceback
+from datetime import datetime
 from jpype import JClass
 import weka.core.jvm as jvm
 from weka.core.classes import JavaObject
 import weka.core.classes as classes
+from weka.core.version import pww_version, with_graph_support, with_plot_support
 
 
 LATEST = "Latest"
 """ Constant for the latest version of a package """
 
+# logging setup
+logger = logging.getLogger("weka.packages")
+logger.setLevel(logging.INFO)
+
 
 class Package(JavaObject):
     """
@@ -96,10 +103,10 @@ class Package(JavaObject):
         :rtype: dict
         """
         if self._metadata is None:
-            map = self.jobject.getPackageMetaData()
+            m = self.jobject.getPackageMetaData()
             self._metadata = dict()
-            for k in map:
-                self._metadata[k] = map[k]
+            for k in m:
+                self._metadata[k] = m[k]
         return self._metadata
 
     @property
@@ -432,13 +439,13 @@ def install_packages(pkges, fail_fast=True, details=False):
 
         if msg is not None:
             if from_repo:
-                print("Failed to install %s/%s:\n%s" % (pkge, version, msg))
+                logger.error("Failed to install %s/%s:\n%s" % (pkge, version, msg))
             else:
-                print("Failed to install %s:\n%s" % (pkge, msg))
+                logger.error("Failed to install %s:\n%s" % (pkge, msg))
             success = False
             result = False
-        elif install_msg is not None:
-            print("Installation message from %s:\n%s" % (pkge, install_msg))
+        elif (install_msg is not None) and (len(install_msg) > 0):
+            logger.info("Installation message from %s:\n%s" % (pkge, install_msg))
 
         result_details[pkge] = {
             "from_repo": from_repo,
@@ -504,48 +511,48 @@ def install_missing_packages(pkges, quiet=False, stop_jvm_and_exit=False):
 
         if not is_installed(pkge):
             if not quiet:
-                print("%s/%s not installed, attempting installation..." % (pkge, version))
+                logger.info("%s/%s not installed, attempting installation..." % (pkge, version))
             if install_package(pkge, version=version):
                 inst = installed_package(pkge)
                 exit_required = True
                 if not quiet:
-                    print("%s/%s successfully installed" % (pkge, inst.version))
+                    logger.info("%s/%s successfully installed" % (pkge, inst.version))
             else:
                 result = False
-                print("Failed to install %s/%s" % (pkge, version))
+                logger.error("Failed to install %s/%s" % (pkge, version))
         else:
             inst = installed_package(pkge)
-            all = all_package(pkge)
+            all_pkg = all_package(pkge)
             install_required = False
-            if (version == LATEST) and (all.version != inst.version):
+            if (version == LATEST) and (all_pkg.version != inst.version):
                 install_required = True
             if (version != LATEST) and (version != inst.version):
                 install_required = True
             if install_required:
                 if not quiet:
                     if version == LATEST:
-                        print("Latest version of %s requested (installed: %s, repository: %s), attempting upgrade" % (pkge, inst.version, all.version))
+                        logger.info("Latest version of %s requested (installed: %s, repository: %s), attempting upgrade" % (pkge, inst.version, all_pkg.version))
                     else:
-                        print("Different version of %s requested (installed: %s, requested: %s), attempting installation" % (pkge, inst.version, version))
+                        logger.info("Different version of %s requested (installed: %s, requested: %s), attempting installation" % (pkge, inst.version, version))
                 if install_package(pkge, version=version):
                     inst = installed_package(pkge)
                     exit_required = True
                     if not quiet:
-                        print("%s/%s successfully installed" % (pkge, inst.version))
+                        logger.info("%s/%s successfully installed" % (pkge, inst.version))
                 else:
                     result = False
-                    print("Failed to install %s/%s" % (pkge, version))
+                    logger.error("Failed to install %s/%s" % (pkge, version))
             else:
                 if not quiet:
-                    print("%s/%s already installed, skipping installation" % (pkge, version))
+                    logger.info("%s/%s already installed, skipping installation" % (pkge, version))
 
     if exit_required:
         if not quiet:
-            print("Package(s) installed, JVM needs to be restarted")
+            logger.info("Package(s) installed, JVM needs to be restarted")
         if stop_jvm_and_exit:
-            print("Stopping JVM...")
+            logger.info("Stopping JVM...")
             jvm.stop()
-            print("Exiting...")
+            logger.info("Exiting...")
             sys.exit(0)
 
     return result, exit_required
@@ -565,8 +572,8 @@ def uninstall_packages(names):
     """
     Uninstalls a package.
 
-    :param name: the names of the package
-    :type name: list
+    :param names: the names of the package
+    :type names: list
     """
     establish_cache()
     for name in names:
@@ -595,6 +602,31 @@ def is_installed(name, version=None):
     return False
 
 
+def is_official_package(name, version=None):
+    """
+    Checks whether the package is an official one.
+
+    :param name: the name of the package to check
+    :type name: str
+    :param version: the specific version to check
+    :type version: str
+    :return: whether an official package or not
+    :rtype: bool
+    """
+    result = False
+    pkgs = all_packages()
+    for pkg in pkgs:
+        if pkg.name == name:
+            if version is None:
+                if pkg.version == version:
+                    result = True
+                    break
+            else:
+                result = True
+                break
+    return result
+
+
 def suggest_package(name, exact=False):
     """
     Suggests package(s) for the given name (classname, package name). Matching can be either exact or just a substring.
@@ -626,6 +658,7 @@ def _output_text(content, format_lambda, output):
     if output is None:
         print("\n".join(formatted))
     else:
+        logger.info("Writing to: %s" % output)
         with open(output, "w") as fp:
             fp.write("\n".join(formatted))
 
@@ -647,6 +680,7 @@ def _output_json(content, content_filter, output):
     if output is None:
         print(json.dumps(filtered, indent=2))
     else:
+        logger.info("Writing to: %s" % output)
         with open(output, "w") as fp:
             json.dump(filtered, fp, indent=2)
 
@@ -696,6 +730,31 @@ def _output_pkg_info(pkgs, args):
             raise Exception("Unhandled type: %s" % args.type)
 
 
+def _freeze_info(output_urls=False, force_urls=False):
+    """
+    Compiles the information for the "freeze" sub-command.
+
+    :param output_urls: whether to output the URLs
+    :type output_urls: bool
+    :param force_urls: whether to always output the URLs
+    :type force_urls: bool
+    :return: the list of dictionaries with the package info
+    :rtype: list
+    """
+    pkgs = installed_packages()
+    result = [pkg.as_dict() for pkg in pkgs]
+    for c in result:
+        output = force_urls or (output_urls and not is_official_package(c["name"], c["version"]))
+        if output:
+            if "url" in c:
+                c["url"] = "|" + c["url"]
+            else:
+                c["url"] = ""
+        else:
+            c["url"] = ""
+    return result
+
+
 def _subcmd_list(args):
     """
     Lists packages (name and version).
@@ -726,14 +785,43 @@ def _subcmd_info(args):
 def _subcmd_install(args):
     """
     Installs one or more packages. Specific versions are suffixed with "==VERSION".
+    A requirements text file can be supplied as well (PKGNAME[==VERSION[|URL]]).
     """
+    if (args.packages is None) and (args.requirements is None):
+        raise Exception("Packages have to be either listed explicitly or provided via a text file (-r/--requirements)!")
+
+    if args.refresh_cache:
+        refresh_cache()
+
     for pkg in args.packages:
         version = LATEST
         if "==" in pkg:
             pkg, version = pkg.split("==")
-        print("Installing: %s/%s" % (pkg, version))
+        logger.info("Installing: %s/%s" % (pkg, version))
         success = install_package(pkg, version)
-        print("  installed successfully" if success else "  failed to install")
+        logger.info("  installed successfully" if success else "  failed to install")
+
+    if args.requirements is not None:
+        logger.info("Processing: %s" % args.requirements)
+        with open(args.requirements, "r") as fp:
+            lines = fp.readlines()
+        for line in lines:
+            pkg = line.strip()
+            if len(pkg) > 0:
+                version = LATEST
+                url = None
+                if "==" in pkg:
+                    pkg, version = pkg.split("==")
+                    if "|" in version:
+                        version, url = version.split("|")
+                if url is None:
+                    logger.info("Installing: %s/%s" % (pkg, version))
+                    success = install_package(pkg, version)
+                    logger.info("  installed successfully" if success else "  failed to install")
+                else:
+                    logger.info("Installing: %s/%s" % (pkg, url))
+                    success = install_package(url)
+                    logger.info("  installed successfully" if success else "  failed to install")
 
 
 def _subcmd_uninstall(args):
@@ -741,12 +829,21 @@ def _subcmd_uninstall(args):
     Uninstalls one or more packages.
     """
     for pkg in args.packages:
-        print("Uninstalling: %s" % pkg)
+        logger.info("Uninstalling: %s" % pkg)
         if is_installed(pkg):
             uninstall_package(pkg)
-            print("  uninstalled")
+            logger.info("  uninstalled")
         else:
-            print("  not installed, skipping")
+            logger.info("  not installed, skipping")
+
+
+def _subcmd_freeze(args):
+    """
+    Outputs the currently installed packages in requirements.txt file format (PKGNAME==VERSION[|URL], one per line).
+    Either to the specified requirements file or to stdout. The URL is only output for "unofficial" packages unless forced.
+    """
+    content = _freeze_info(output_urls=args.output_urls, force_urls=args.force_urls)
+    _output_text(content, lambda x: "%s==%s%s" % (x["name"], x["version"], x["url"]), args.requirements)
 
 
 def _subcmd_suggest(args):
@@ -765,6 +862,9 @@ def _subcmd_suggest(args):
 
 
 def _subcmd_is_installed(args):
+    """
+    Checks whether the package(s) are installed.
+    """
     content = []
     for name in args.name:
         version = None
@@ -784,6 +884,81 @@ def _subcmd_is_installed(args):
         _output_json(content, None, args.output)
 
 
+def _subcmd_bootstrap(args):
+    """
+    Generates a python script that bootstraps pww3 and currently installed packages
+    on another computer.
+    """
+    lines = list()
+
+    # general info
+    lines.append("# python-weka-wrapper3 - bootstrap")
+    lines.append("# Generated: %s" % str(datetime.now()))
+    lines.append("")
+
+    # logging
+    lines.append("# 1. configure logging")
+    lines.append("import logging")
+    lines.append("logging.basicConfig()")
+    lines.append('logger = logging.getLogger("bootstrap")')
+    lines.append("logger.setLevel(logging.INFO)")
+    lines.append("")
+
+    # pww3
+    additional = list()
+    if with_graph_support():
+        additional.append("graphs")
+    if with_plot_support():
+        additional.append("plots")
+    if len(additional) > 0:
+        additional_modules = "[%s]" % ",".join(additional)
+    else:
+        additional_modules = ""
+    lines.append("# 2. install python-weka-wrapper3")
+    lines.append('logger.info("Installing python-weka-wrapper3...")')
+    lines.append("import subprocess")
+    lines.append("import sys")
+    version = pww_version()
+    lines.append("pkgs = [")
+    lines.append('    "python-weka-wrapper3%s==%s"' % (additional_modules, version))
+    lines.append("]")
+    lines.append('subprocess.check_call([sys.executable, "-m", "pip", "install", " ".join(pkgs)])')
+    lines.append("")
+
+    # packages
+    content = _freeze_info(output_urls=True, force_urls=args.force_urls)
+    if len(content) > 0:
+        lines.append("# 3. install Weka packages")
+        lines.append('logger.info("Installing Weka packages...")')
+        lines.append("import weka.core.jvm as jvm")
+        lines.append("from weka.core.packages import install_package")
+        lines.append("jvm.start(packages=True)")
+        lines.append("")
+        for c in content:
+            lines.append("# %s" % c["name"])
+            if ("url" in c) and (len(c["url"]) > 0):
+                url = c["url"]
+                if "|" in url:
+                    _, url = url.split("|")
+                lines.append('logger.info("%s/%s")' % (c["name"], url))
+                lines.append('install_package("%s")' % url)
+            else:
+                lines.append('logger.info("%s/%s")' % (c["name"], c["version"]))
+                lines.append('install_package("%s", "%s")' % (c["name"], c["version"]))
+        lines.append("")
+        lines.append("jvm.stop()")
+
+    # output
+    script = "\n".join(lines)
+    if args.output is None:
+        print(script)
+    else:
+        logger.info("Writing script to: %s" % args.output)
+        with open(args.output, "w") as fp:
+            fp.write(script)
+
+
+
 def main(args=None):
     """
     Performs the specified package operation from the command-line. Calls JVM start/stop automatically.
@@ -817,7 +992,9 @@ def main(args=None):
 
     # install
     parser = sub_parsers.add_parser("install", help="For installing one or more packages")
-    parser.add_argument("packages", nargs="+", help="the name of the package(s) to install, append '==VERSION' to pin to a specific version")
+    parser.add_argument("packages", nargs="*", help="the name of the package(s) to install, append '==VERSION' to pin to a specific version")
+    parser.add_argument("-r", "--requirements", metavar="FILE", default=None, help="the text file with packages to install (one per line, format: PKGNAME[==VERSION[|URL]])")
+    parser.add_argument("--refresh-cache", dest="refresh_cache", action="store_true", help="whether to refresh the package cache")
     parser.set_defaults(func=_subcmd_install)
 
     # uninstall/remove
@@ -825,6 +1002,13 @@ def main(args=None):
     parser.add_argument("packages", nargs="+", help="the name of the package(s) to uninstall")
     parser.set_defaults(func=_subcmd_uninstall)
 
+    # freeze
+    parser = sub_parsers.add_parser("freeze", help="For outputting list of installed packages")
+    parser.add_argument("-r", "--requirements", metavar="FILE", default=None, help="the text file to store the package/version pairs in (one per line, format: PKGNAME[==VERSION])")
+    parser.add_argument("-u", "--output_urls", action="store_true", help="whether to output the download URL for unofficial packages (appends '|URL')")
+    parser.add_argument("-f", "--force_urls", action="store_true", help="forces the output of the URLs for all packages, not just unofficial ones")
+    parser.set_defaults(func=_subcmd_freeze)
+
     # suggest
     parser = sub_parsers.add_parser("suggest", help="For suggesting packages that contain the specified class")
     parser.add_argument("classname", nargs=1, help="the classname to suggest packages for")
@@ -840,6 +1024,12 @@ def main(args=None):
     parser.add_argument("-o", "--output", metavar="FILE", default=None, help="the file to store the output in, uses stdout if not supplied")
     parser.set_defaults(func=_subcmd_is_installed)
 
+    # bootstrap
+    parser = sub_parsers.add_parser("bootstrap", help="Generates Python script for recreating current pww3 environment.")
+    parser.add_argument("-f", "--force_urls", action="store_true", help="forces the install from URLs, not just unofficial ones")
+    parser.add_argument("-o", "--output", metavar="FILE", default=None, help="the file to store the Python script in, otherwise outputs it on stdout")
+    parser.set_defaults(func=_subcmd_bootstrap)
+
     parsed = main_parser.parse_args(args=args)
     # no action chosen? show help
     if not hasattr(parsed, "func"):
