@@ -27,6 +27,7 @@ from weka.core.dataset import Instances, Instance
 
 # logging setup
 logger = logging.getLogger("weka.datagenerators")
+logger.setLevel(logging.INFO)
 
 
 class DataGenerator(OptionHandler):
@@ -199,7 +200,7 @@ def main(args=None):
 
     jvm.start(jars, max_heap_size=parsed.heap, packages=True)
 
-    logger.debug("Commandline: " + join_options(sys.argv[1:]))
+    logger.info("Commandline: " + join_options(sys.argv[1:]))
 
     try:
         generator = DataGenerator(classname=parsed.datagenerator)
